# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from commonconf import settings
from uw_r25.spaces import get_space_by_id, get_spaces
from restclients_core.exceptions import DataFailureException
from uw_r25.util import fdao_r25_override


@fdao_r25_override
class R25TestSpaces(TestCase):

    def test_space_by_id(self):
        space = get_space_by_id("1000")
        self.assertEquals(space.space_id, "1000", "space_id")
        self.assertEquals(space.name, "ACC 120", "name")
        self.assertEquals(space.formal_name, "Smith Hall", "formal_name")

    def test_all_spaces(self):
        spaces = get_spaces()
        self.assertEquals(len(spaces), 3, "space count")



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from restclients_core import models


class Event(models.Model):
    DRAFT_STATE = "0"
    TENTATIVE_STATE = "1"
    CONFIRMED_STATE = "2"
    SEALED_STATE = "3"
    DENIED_STATE = "4"
    CANCELLED_STATE = "99"

    STATE_CHOICES = (
        (DRAFT_STATE, "Draft"),
        (TENTATIVE_STATE, "Tentative"),
        (CONFIRMED_STATE, "Confirmed"),
        (SEALED_STATE, "Sealed"),
        (DENIED_STATE, "Denied"),
        (CANCELLED_STATE, "Cancelled"),
    )

    event_id = models.IntegerField()
    alien_uid = models.CharField(max_length=100, null=True)
    name = models.CharField(max_length=100)
    title = models.CharField(max_length=100)
    start_date = models.DateField()
    end_date = models.DateField()
    state = models.CharField(max_length=2, choices=STATE_CHOICES)
    parent_id = models.IntegerField(null=True)
    cabinet_id = models.IntegerField(null=True)
    cabinet_name = models.CharField(max_length=100, null=True)

    def state_name(self):
        return dict(self.STATE_CHOICES)[self.state]

    def parent(self):
        if not hasattr(self, "_parent"):
            self._parent = None
            if self.parent_id is not None:
                from uw_r25.events import get_event_by_id
                self._parent = get_event_by_id(self.parent_id)
        return self._parent

    def children(self):
        if not hasattr(self, "_children"):
            from uw_r25.events import get_events
            self._children = get_events(parent_id=self.event_id)
        return self._children

    def cabinet(self):
        if self.cabinet_id is not None:
            if self.cabinet_id == self.event_id:
                return self
            else:
                from uw_r25.events import get_event_by_id
                return get_event_by_id(self.cabinet_id)

    class Meta:
        db_table = "restclients_r25_event"


class Space(models.Model):
    space_id = models.IntegerField()
    name = models.CharField(max_length=100)
    formal_name = models.CharField(max_length=200)

    class Meta:
        db_table = "restclients_r25_space"


class Reservation(models.Model):
    STANDARD_STATE = "1"
    EXCEPTION_STATE = "2"
    WARNING_STATE = "3"
    OVERRIDE_STATE = "4"
    CANCELLED_STATE = "99"

    STATE_CHOICES = (
        (STANDARD_STATE, "Standard"),
        (EXCEPTION_STATE, "Exception"),
        (WARNING_STATE, "Warning"),
        (OVERRIDE_STATE, "Override"),
        (CANCELLED_STATE, "Cancelled"),
    )

    reservation_id = models.IntegerField()
    state = models.CharField(max_length=2, choices=STATE_CHOICES)
    start_datetime = models.DateTimeField()
    end_datetime = models.DateTimeField()
    event_id = models.IntegerField()
    event_name = models.CharField(max_length=64)
    profile_name = models.CharField(max_length=32)
    contact_name = models.CharField(max_length=64)
    contact_email = models.CharField(max_length=64)

    def state_name(self):
        return dict(self.STATE_CHOICES)[self.state]

    class Meta:
        db_table = "restclients_r25_reservation"


class BindingReservation(models.Model):
    bound_reservation_id = models.IntegerField()
    primary_reservation = models.IntegerField()
    name = models.CharField(max_length=200)
    bound_event_id = models.IntegerField()

    class Meta:
        db_table = "restclients_r25_binding_reservation"



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import re
from os.path import abspath, dirname
from restclients_core.dao import DAO


class R25_DAO(DAO):
    def service_name(self):
        return 'r25'

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]

    def _custom_headers(self, method, url, headers, body):
        basic_auth = self.get_service_setting('BASIC_AUTH')
        if basic_auth is not None:
            return {"Authorization": "Basic {}".format(basic_auth)}



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase, skipIf
from commonconf import settings
from uw_r25.events import get_event_by_id, get_events
from uw_r25.util import fdao_r25_override
from restclients_core.exceptions import DataFailureException


@fdao_r25_override
class R25TestEvents(TestCase):

    def test_event_by_id(self):
        event = get_event_by_id("100000")
        self.assertEquals(event.event_id, "100000", "event_id")
        self.assertEquals(event.name, "BOTHELL WINTER 2013 CABINET", "name")
        self.assertEquals(event.title, "BOTHELL WINTER 2013 CABINET", "title")
        self.assertEquals(event.alien_uid, None, "alien_uid")
        self.assertEquals(event.start_date, "2013-01-01", "start_date")
        self.assertEquals(event.end_date, "2013-03-28", "end_date")
        self.assertEquals(event.state, "1", "state")
        self.assertEquals(event.parent_id, None, "parent_id")
        self.assertEquals(event.cabinet_id, event.event_id, "cabinet_id")
        self.assertEquals(event.cabinet_name, event.name, "cabinet_name")
        self.assertEquals(event.state_name(), "Tentative", "state_name")
        self.assertEquals(len(event.reservations), 1, "reservations")
        self.assertEquals(
            len(event.binding_reservations), 1, "binding_reservations")

    def test_parent_event(self):
        event = get_event_by_id("100002")
        parent = event.parent()
        self.assertEquals(parent.event_id, event.parent_id, "parent_id")

        # No parent event
        parent2 = parent.parent()
        self.assertEquals(parent2, None, "parent_id")

    def test_child_events(self):
        event = get_event_by_id("100000")
        children = event.children()
        self.assertEquals(len(children), 1, "child event count")

        # No child events
        child = children[0]
        children = child.children()
        self.assertEquals(len(children), 0, "child event count")

    def test_cabinet_event(self):
        event = get_event_by_id("100002")
        cabinet = event.cabinet()
        self.assertEquals(cabinet.event_id, event.cabinet_id, "cabinet_id")

        # cabinet is self
        cabinet2 = cabinet.cabinet()
        self.assertEquals(cabinet2.event_id, cabinet.event_id, "cabinet_id")

    def test_all_events(self):
        events = get_events()
        self.assertEquals(len(events), 3, "event count")



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from uw_r25.models import Reservation
from uw_r25 import nsmap, get_resource
from uw_r25.spaces import space_reservation_from_xml
from urllib.parse import urlencode


def get_reservation_by_id(reservation_id):
    url = "reservation.xml?rsrv_id={}".format(reservation_id)
    return reservations_from_xml(get_resource(url))[0]


def get_reservations(**kwargs):
    """
    Return a list of reservations matching the passed filter.
    Supported kwargs are listed at
    http://knowledge25.collegenet.com/display/WSW/reservations.xml
    """
    kwargs["scope"] = "extended"
    url = "reservations.xml"
    if len(kwargs):
        url += "?{}".format(urlencode(kwargs))

    return reservations_from_xml(get_resource(url))


def reservations_from_xml(tree):
    try:
        profile_name = tree.xpath("r25:profile_name", namespaces=nsmap)[0].text
    except Exception:
        profile_name = None

    reservations = []
    for node in tree.xpath("r25:reservation", namespaces=nsmap):
        reservation = Reservation()
        reservation.reservation_id = node.xpath("r25:reservation_id",
                                                namespaces=nsmap)[0].text
        reservation.start_datetime = node.xpath("r25:reservation_start_dt",
                                                namespaces=nsmap)[0].text
        reservation.end_datetime = node.xpath("r25:reservation_end_dt",
                                              namespaces=nsmap)[0].text
        reservation.state = node.xpath("r25:reservation_state",
                                       namespaces=nsmap)[0].text
        if profile_name:
            reservation.profile_name = profile_name
        else:
            reservation.profile_name = node.xpath("r25:profile_name",
                                                  namespaces=nsmap)[0].text

        try:
            pnode = node.xpath("r25:space_reservation", namespaces=nsmap)[0]
            reservation.space_reservation = space_reservation_from_xml(pnode)
        except IndexError:
            reservation.space_reservation = None

        try:
            enode = node.xpath("r25:event", namespaces=nsmap)[0]
            reservation.event_id = enode.xpath("r25:event_id",
                                               namespaces=nsmap)[0].text
            reservation.event_name = enode.xpath("r25:event_name",
                                                 namespaces=nsmap)[0].text

            rnode = enode.xpath("r25:role", namespaces=nsmap)[0]
            cnode = rnode.xpath("r25:contact", namespaces=nsmap)[0]
            reservation.contact_name = cnode.xpath("r25:contact_name",
                                                   namespaces=nsmap)[0].text
            try:
                anode = cnode.xpath("r25:address", namespaces=nsmap)[0]
                reservation.contact_email = anode.xpath(
                    "r25:email", namespaces=nsmap)[0].text
            except IndexError:
                reservation.contact_email = None

        except IndexError:
            enode = tree.getparent()
            reservation.event_id = enode.xpath("r25:event_id",
                                               namespaces=nsmap)[0].text
            reservation.event_name = enode.xpath("r25:event_name",
                                                 namespaces=nsmap)[0].text

        reservations.append(reservation)

    return reservations



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_r25.dao import R25_DAO
from commonconf import override_settings


class R25TestDao(TestCase):

    def test_custom_headers(self):
        self.assertFalse(R25_DAO()._custom_headers('GET', '/', {}, None))
        with override_settings(RESTCLIENTS_R25_BASIC_AUTH='b64here'):
            self.assertEquals(
                R25_DAO()._custom_headers('GET', '/', {}, None),
                {'Authorization': 'Basic b64here'}
            )



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from uw_r25.models import Event, BindingReservation
from uw_r25 import nsmap, get_resource
from uw_r25.reservations import reservations_from_xml
from urllib.parse import urlencode, quote


def get_event_by_id(event_id):
    url = "event.xml?event_id={}".format(event_id)
    return events_from_xml(get_resource(url))[0]


def get_event_by_alien_id(alien_id):
    url = "event.xml?alien_uid={}".format(quote(alien_id))
    event = events_from_xml(get_resource(url))
    return event[0] if event else None


def get_events(**kwargs):
    """
    Return a list of events matching the passed filter.
    Supported kwargs are listed at
    http://knowledge25.collegenet.com/display/WSW/events.xml
    """
    url = "events.xml"
    if len(kwargs):
        url += "?{}".format(urlencode(kwargs))

    return events_from_xml(get_resource(url))


def events_from_xml(tree):
    events = []
    for node in tree.xpath("r25:event", namespaces=nsmap):
        event = Event()
        event.event_id = node.xpath("r25:event_id", namespaces=nsmap)[0].text
        event.alien_uid = node.xpath("r25:alien_uid", namespaces=nsmap)[0].text
        event.name = node.xpath("r25:event_name", namespaces=nsmap)[0].text
        event.title = node.xpath("r25:event_title", namespaces=nsmap)[0].text
        event.start_date = node.xpath("r25:start_date",
                                      namespaces=nsmap)[0].text
        event.end_date = node.xpath("r25:end_date", namespaces=nsmap)[0].text
        event.state = node.xpath("r25:state", namespaces=nsmap)[0].text
        event.parent_id = node.xpath("r25:parent_id", namespaces=nsmap)[0].text
        event.cabinet_id = node.xpath("r25:cabinet_id",
                                      namespaces=nsmap)[0].text
        event.cabinet_name = node.xpath("r25:cabinet_name",
                                        namespaces=nsmap)[0].text

        event.binding_reservations = []
        event.reservations = []
        for pnode in node.xpath("r25:profile", namespaces=nsmap):
            event.binding_reservations += binding_reservations_from_xml(pnode)
            event.reservations += reservations_from_xml(pnode)
        events.append(event)

    return events


def binding_reservations_from_xml(tree):
    binding_reservations = []
    for node in tree.xpath("//r25:binding_reservation", namespaces=nsmap):
        bind_res = BindingReservation()
        bind_res.bound_reservation_id = node.xpath("r25:bound_reservation_id",
                                                   namespaces=nsmap)[0].text
        bind_res.primary_reservation = node.xpath("r25:primary_reservation",
                                                  namespaces=nsmap)[0].text
        bind_res.name = node.xpath("r25:bound_name", namespaces=nsmap)[0].text
        bind_res.bound_event_id = node.xpath("r25:bound_event_id",
                                             namespaces=nsmap)[0].text
        binding_reservations.append(bind_res)

    return binding_reservations



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from commonconf import settings
from uw_r25.reservations import get_reservation_by_id, get_reservations
from restclients_core.exceptions import DataFailureException
from uw_r25.util import fdao_r25_override


@fdao_r25_override
class R25TestReservations(TestCase):

    def test_reservation_by_id(self):
        pass

    def test_get_reservations(self):
        pass



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from restclients_core.util.decorators import use_mock
from uw_r25.dao import R25_DAO

fdao_r25_override = use_mock(R25_DAO())



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from uw_r25.models import Space
from uw_r25 import nsmap, get_resource
from urllib.parse import urlencode


def get_space_by_id(space_id):
    url = "space.xml?space_id={}".format(space_id)
    return spaces_from_xml(get_resource(url))[0]


def get_spaces(**kwargs):
    """
    Return a list of reservations matching the passed filter.
    Supported kwargs are listed at
    http://knowledge25.collegenet.com/display/WSW/spaces.xml
    """
    url = "spaces.xml"
    if len(kwargs):
        url += "?{}".format(urlencode(kwargs))

    return spaces_from_xml(get_resource(url))


def spaces_from_xml(tree):
    spaces = []
    for node in tree.xpath("//r25:space", namespaces=nsmap):
        space = space_from_xml(node)
        spaces.append(space)

    return spaces


def space_from_xml(tree):
    space = Space()
    space.space_id = tree.xpath("r25:space_id", namespaces=nsmap)[0].text
    space_detail_from_xml(space, tree)
    return space


def space_reservation_from_xml(tree):
    space = Space()
    space.space_id = tree.xpath("r25:space_id", namespaces=nsmap)[0].text
    node = tree.xpath("r25:space", namespaces=nsmap)[0]
    space_detail_from_xml(space, node)
    return space


def space_detail_from_xml(space, tree):
    space.name = tree.xpath("r25:space_name", namespaces=nsmap)[0].text
    space.formal_name = tree.xpath("r25:formal_name",
                                   namespaces=nsmap)[0].text



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from uw_r25.dao import R25_DAO
from restclients_core.exceptions import DataFailureException
from lxml import etree


nsmap = {"r25": "http://www.collegenet.com/r25",
         "xhtml": "http://www.w3.org/1999/xhtml"}


def get_resource(url):
    """
    Issue a GET request to R25 with the given url
    and return a response as an etree element.
    """

    instance = R25_DAO().get_service_setting('INSTANCE')
    if instance is not None:
        url = "/r25ws/wrd/{}/run/{}".format(instance, url)
    else:
        url = "/r25ws/servlet/wrd/run/{}".format(url)

    response = R25_DAO().getURL(url, {"Accept": "text/xml"})
    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    tree = etree.fromstring(response.data.strip())

    # XHTML response is an error response
    xhtml = tree.xpath("//xhtml:html", namespaces=nsmap)
    if len(xhtml):
        raise DataFailureException(url, 500, response.data)

    return tree
